"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var inflection_1 = require("inflection");
var env = require("./env");
function parseComponent(component) {
    return prepareComponent([], component, 0, 0, false);
}
exports.parseComponent = parseComponent;
function escapeBoundExpression(value) {
    if (value === void 0) { value = ''; }
    return value.replace(/\$/g, '&#36;').replace(/{/g, '&#123;');
}
function prepareComponent(pageProperties, component, indexByType, index, interpolate) {
    if (indexByType === void 0) { indexByType = 0; }
    if (index === void 0) { index = 0; }
    if (interpolate === void 0) { interpolate = true; }
    var properties = [];
    var components = [];
    var encode = interpolate ? function (value) { return env.interpolateAsAttribute(pageProperties, value); } : escapeBoundExpression;
    Object.keys(component).filter(function (key) { return !/^(section|content|type|index|row|col|events|height)$/.test(key); })
        .forEach(function (name) {
        var value = component[name];
        if (Array.isArray(value)) {
            value.forEach(function (item) {
                var type = item.type;
                var child = prepareComponent(pageProperties, Object.assign({}, item, {
                    content: item.template ? "<ng-template let-data>\n" + item.template + "\n</ng-template>" : '',
                    type: component.type + "-" + inflection_1.singularize(name)
                }));
                if (type !== undefined) {
                    child.properties.push({ name: 'type', value: type });
                }
                if (item.template) {
                    child.properties = child.properties.filter(function (property) { return property.name != 'template'; });
                }
                child.name = null;
                components.push(child);
            });
        }
        else {
            var bound = env.isBoundExpression(value);
            value = bound ? env.interpolateAsAttribute(pageProperties, value) : value;
            if (bound && value.startsWith('{{') && value.endsWith('}}')) {
                value = value.replace('{{', '').replace('}}', '');
            }
            properties.push({
                name: name,
                bound: bound,
                value: value
            });
        }
    });
    var name = component.name || "" + component.type + indexByType;
    var events = Object.keys(component.events || {}).map(function (event) {
        var value = "" + name + env.type(event);
        return {
            name: event,
            value: value,
            actions: component.events[event]
        };
    });
    var content = component.content;
    return {
        type: component.type,
        name: name,
        row: component.row,
        col: component.col,
        height: component.height,
        content: encode(content),
        events: events,
        properties: properties,
        components: components
    };
}
exports.prepareComponent = prepareComponent;
//# sourceMappingURL=Component.js.map