export interface Events {
    [name: string]: Actions;
}
export declare type Action = Invoke | Set | Filter | Navigate | Back | Notify | Execute | Popup;
export declare type Actions = Array<Action>;
export interface Execute {
    type: 'execute';
    code: string;
}
export interface Invoke {
    type: 'invoke';
    name: string;
    parameters?: Array<Parameter>;
    then: Actions;
    error: Actions;
}
export interface Notify {
    type: 'notify';
    severity: string;
    summary: string;
    detail: string;
}
export interface Parameter {
    name?: string;
    value: string;
}
export interface Set {
    type: 'set';
    name: string;
    value: string;
}
export interface Filter {
    type: 'filter';
    name: string;
    operator: string;
    property: string;
    value: string;
    then: Actions;
}
export interface Navigate {
    type: 'navigate';
    path: string;
    parameters?: Array<Parameter>;
}
export interface Back {
    type: 'back';
}
export interface Popup {
    type: 'popup';
    path: string;
    parameters?: Array<Parameter>;
    width?: number;
    height?: number;
}
export declare function isPopup(action: Action): action is Popup;
export declare function isNavigate(action: Action): action is Navigate;
export declare function isNavigateBack(action: Action): action is Back;
export declare function isFilter(action: Action): action is Filter;
export declare function isInvoke(action: Action): action is Invoke;
export declare function isSet(action: Action): action is Set;
export declare function isNotify(action: Action): action is Notify;
export declare function isExecute(action: Action): action is Execute;
