"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var child_process = require("child_process");
var fs_1 = require("./fs");
function linkNodeModules(path, node_modules) {
    return removeLink(path).then(function () {
        return fs_1.symlink(node_modules, path_1.join(path, 'client', 'node_modules'), 'junction')
            .then(function () {
            // console.log('symlink created');
        }, function (err) {
            if (err.code == 'EEXIST') {
                // console.log('symlink exists');
            }
            else {
                // console.error('symlink creating failed', err);
                throw err;
            }
        });
    });
}
function removeLink(path) {
    var dir = path_1.join(path, 'client', 'node_modules');
    return fs_1.lstat(dir).then(function (stats) {
        if (stats.isSymbolicLink()) {
            return fs_1.unlink(dir).then(function () {
            }, function (err) {
                if (err.code != 'ENOENT') {
                    throw err;
                }
            });
        }
    }, function (err) {
        if (err.code != 'ENOENT') {
            throw err;
        }
    });
}
function options(application) {
    var options = [
        require.resolve('@angular/cli/bin/ng'),
        'serve',
        '--open',
        '--port',
        "" + application.port,
        '--progress',
        'false'
    ];
    if (application.https) {
        options.push('--ssl', 'true');
    }
    return options;
}
function build(_a) {
    var directory = _a.directory, application = _a.application, modules = _a.modules, onStart = _a.onStart, onExit = _a.onExit, onError = _a.onError;
    linkNodeModules(directory, modules)
        .then(function () {
        var child = child_process.spawn(process.execPath, options(application), { cwd: path_1.join(directory, 'client'), env: Object.assign({ ELECTRON_RUN_AS_NODE: 1 }, process.env) });
        onStart(child);
        child.on('error', function (error) {
            removeLink(directory);
            onError();
        });
        child.on('exit', function (code) {
            removeLink(directory);
            if (code > 0) {
                onError();
            }
            else {
                onExit();
            }
        });
        child.stdout.on('data', function (data) {
            console.log('ng-cli:', data.toString());
        });
        child.stderr.on('data', function (data) {
            console.log('ng-cli:', data.toString());
        });
    });
}
exports.build = build;
//# sourceMappingURL=build.js.map