"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path_1 = require("path");
var rimraf = require("rimraf");
var mkdirp = require("mkdirp");
function ensure() {
    var dirs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        dirs[_i] = arguments[_i];
    }
    return dirs.reduce(function (path, dir) {
        path = path_1.join(path, dir);
        if (!fs.existsSync(path)) {
            fs.mkdirSync(path);
        }
        return path;
    }, '');
}
exports.ensure = ensure;
function promisify(func) {
    return function () {
        var parameters = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            parameters[_i] = arguments[_i];
        }
        return new Promise(function (resolve, reject) {
            func.apply(void 0, parameters.concat([function (error, result) {
                    if (error) {
                        reject(error);
                    }
                    else {
                        resolve(result);
                    }
                }]));
        });
    };
}
var ensurePath = promisify(mkdirp);
function cp(source, dest) {
    return ensurePath(dest)
        .then(function () { return exports.readdir(source); })
        .then(function (entries) {
        return Promise.all(entries.map(function (entry) {
            var sourcePath = path_1.join(source, entry);
            var destPath = path_1.join(dest, entry);
            return dirExists(sourcePath)
                .then(function (exists) {
                if (exists) {
                    return makedir(destPath).then(function () { return cp(sourcePath, destPath); });
                }
                else {
                    return copy(sourcePath, destPath);
                }
            });
        }));
    });
}
exports.cp = cp;
function makedir(source) {
    return exports.mkdir(source).then(function () { }, function (err) {
        if (err.code != 'EEXIST') {
            throw err;
        }
    });
}
exports.makedir = makedir;
function copy(source, target) {
    return new Promise(function (resolve, reject) {
        var rd = fs.createReadStream(source);
        rd.on('error', reject);
        var wr = fs.createWriteStream(target);
        wr.on('error', reject);
        wr.on('close', resolve);
        rd.pipe(wr);
    });
}
exports.copy = copy;
function dirExists(source) {
    return exports.lstat(source).then(function (stats) { return stats.isDirectory(); }, function () { return false; });
}
exports.dirExists = dirExists;
function fileExists(source) {
    return exports.lstat(source).then(function (stats) { return stats.isFile(); }, function () { return false; });
}
exports.fileExists = fileExists;
exports.readdir = promisify(fs.readdir);
exports.readFile = promisify(fs.readFile);
exports.writeFile = promisify(fs.writeFile);
exports.rm = promisify(rimraf);
exports.mkdir = promisify(fs.mkdir);
exports.rename = promisify(fs.rename);
exports.lstat = promisify(fs.lstat);
exports.symlink = promisify(fs.symlink);
exports.unlink = promisify(fs.unlink);
//# sourceMappingURL=fs.js.map