"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path_1 = require("path");
var child_process = require("child_process");
var lodash_1 = require("lodash");
var dotenv = require("dotenv");
var Component_1 = require("./Component");
var Event_1 = require("./Event");
var DataSource_1 = require("./DataSource");
var env = require("./env");
var renderTemplate_1 = require("./renderTemplate");
var fs_1 = require("./fs");
var securityDataSource_1 = require("./securityDataSource");
__export(require("./toolbox"));
__export(require("./bootstrap"));
__export(require("./build"));
__export(require("./securityDataSource"));
function updateFile(path, content) {
    return fs_1.fileExists(path).then(function (exists) {
        return exists ? fs_1.readFile(path) : null;
    }).then(function (original) {
        if (original != content) {
            return fs_1.writeFile(path, content);
        }
    });
}
function styles(path, application) {
    return [
        require.resolve('primeng/resources/primeng.min.css'),
        path_1.join(path, 'client', 'src', 'assets', 'css', application.theme + ".css")
    ];
}
exports.styles = styles;
function copyThemeDir(path, source, name) {
    return fs_1.cp(source, path_1.join(path, 'client', 'src', 'assets', name));
}
exports.copyThemeDir = copyThemeDir;
function setConfigValue(path, key, value) {
    var file = path_1.join(path, '.env');
    return fs_1.readFile(file).then(function (data) {
        return dotenv.parse(data);
    }, function () { return ({}); }).then(function (config) {
        config[key] = value;
        var lines = Object.keys(config).map(function (key) {
            return key + "=\"" + config[key] + "\"";
        });
        return fs_1.writeFile(file, lines.join('\n'));
    });
}
exports.setConfigValue = setConfigValue;
function getConfigValue(path, key) {
    var file = path_1.join(path, '.env');
    return fs_1.readFile(file).then(function (data) {
        var config = dotenv.parse(data);
        return config[key];
    }, function () { return null; });
}
exports.getConfigValue = getConfigValue;
function getConnectionString(path, key) {
    return getConfigValue(path, "ConnectionStrings__" + key);
}
exports.getConnectionString = getConnectionString;
function setConnectionString(path, key, value) {
    return setConfigValue(path, "ConnectionStrings__" + key, value);
}
exports.setConnectionString = setConnectionString;
function saveSSL(path, key, cert) {
    var client = fs_1.ensure(path, 'client');
    var ssl = fs_1.ensure(client, 'ssl');
    return Promise.all([
        fs_1.writeFile(path_1.join(ssl, 'server.key'), key),
        fs_1.writeFile(path_1.join(ssl, 'server.crt'), cert)
    ]);
}
exports.saveSSL = saveSSL;
function generate(path, application, isRunning) {
    if (isRunning === void 0) { isRunning = false; }
    if (application.security && !application.security.loginPage) {
        return Promise.reject('Security is enabled bu no login page is specified.');
    }
    var dataSources = application.dataSources.map(DataSource_1.swaggerToDataSource);
    var pages = application.pages;
    var client = fs_1.ensure(path, 'client');
    var server = fs_1.ensure(path, 'server');
    var src = fs_1.ensure(client, 'src');
    var assets = fs_1.ensure(src, 'assets');
    var app = fs_1.ensure(src, 'app');
    var environments = fs_1.ensure(src, 'environments');
    return Promise.all([
        copyFile(path, 'gitignore', '.gitignore', isRunning),
        copy(app, 'plus-query-encoder.ts', isRunning),
        copy(client, '.angular-cli.json', isRunning),
        writeTemplate(path_1.join(client, 'package.json'), 'package.json.ejs', application),
        copy(src, 'tsconfig.json', isRunning),
        copy(src, 'polyfills.ts', isRunning),
        copy(src, 'main.ts', isRunning),
        copy(src, 'favicon.ico', isRunning),
        writeTemplate(path_1.join(src, 'styles.css'), 'styles.css.ejs', application),
        writeTemplate(path_1.join(environments, 'environment.ts'), 'environment.ts.ejs', { application: application, dataSources: dataSources }),
        writeTemplate(path_1.join(environments, 'environment.prod.ts'), 'environment.prod.ts.ejs', { application: application, dataSources: dataSources }),
        outputODataServices(server, application, dataSources, isRunning),
        application.security ? outputSecurity(app, application) : Promise.resolve(true),
        outputDataSources(app, application, dataSources),
        outputPages(app, application, pages, dataSources),
        writeAppModule(app, application, dataSources, pages),
        writeTemplate(path_1.join(src, 'index.html'), 'index.html.ejs', application),
        writeTemplate(path_1.join(app, 'app.routes.ts'), 'app.routes.ts.ejs', Object.assign({ security: null }, application, {
            pages: pages,
            parameters: pageParameters(pages)
        })),
        writeTemplate(path_1.join(app, 'app.component.ts'), 'app.component.ts.ejs', Object.assign({ security: null }, application, {
            pages: pages
        })),
        writeTemplate(path_1.join(app, 'app.component.html'), 'app.component.html.ejs', Object.assign({ logo: null, security: null }, application, {
            pages: pages
        }))
    ]);
}
exports.generate = generate;
function startServer(_a) {
    var directory = _a.directory, onStart = _a.onStart, onError = _a.onError, onExit = _a.onExit, onSuccess = _a.onSuccess;
    var cwd = path_1.join(directory, 'server');
    var success = false;
    var env = Object.assign({ ASPNETCORE_ENVIRONMENT: 'Development' }, process.env);
    if (env.hasOwnProperty('Path') && env.hasOwnProperty('PATH')) {
        delete env.Path;
    }
    fs_1.fileExists(path_1.join(cwd, 'obj', 'project.assets.json'))
        .then(function (exists) {
        if (!exists) {
            console.log('dotnet: ', 'Restoring packages ...');
            return new Promise(function (resolve, reject) {
                var child = child_process.spawn('dotnet', ['restore'], { cwd: cwd, env: env });
                onStart(child);
                child.on('error', reject);
                child.on('exit', function (code) {
                    if (code > 0) {
                        reject();
                    }
                    else {
                        resolve();
                    }
                });
                child.stderr.on('data', function (data) {
                    console.log('dotnet:', data.toString());
                });
                child.stdout.on('data', function (data) {
                    console.log('dotnet:', data.toString());
                });
            });
        }
    })
        .then(function () {
        var child = child_process.spawn('dotnet', ['run'], { cwd: cwd, env: env });
        onStart(child);
        child.on('error', onError);
        child.on('exit', function (code) {
            if (code > 0) {
                onError();
            }
            else {
                onExit();
            }
        });
        child.stderr.on('data', function (data) {
            console.log('dotnet:', data.toString());
        });
        child.stdout.on('data', function (data) {
            var message = data.toString();
            console.log('dotnet:', message);
            if (message.includes('Application started') && !success) {
                success = true;
                onSuccess();
            }
        });
    }, onError);
}
exports.startServer = startServer;
function supportsServer() {
    return new Promise(function (resolve, reject) {
        child_process.exec('dotnet --version', function (err) {
            if (err) {
                console.error(err);
                return reject('MSSQL server support requires .NET Core.');
            }
            resolve();
        });
    });
}
exports.supportsServer = supportsServer;
function outputODataServices(path, application, dataSources, isRunning) {
    var odataDataSources = dataSources.filter(function (dataSource) { return dataSource.connector == 'mssql'; });
    if (odataDataSources.length) {
        fs_1.ensure(path);
        fs_1.ensure(path_1.join(path, 'Models'));
        fs_1.ensure(path_1.join(path, 'Controllers'));
        fs_1.ensure(path_1.join(path, 'Data'));
        var models = odataDataSources.map(function (dataSource) { return outputODataModels(path, application.name, dataSource); });
        var controllers = odataDataSources.map(function (dataSource) { return outputODataControllers(path, application.name, dataSource); });
        var contexts = odataDataSources.map(function (dataSource) { return outputODataContext(path, application, dataSource); });
        return Promise.all(models.concat(controllers, contexts, [
            application.security ? outputIdentity(path, application) : Promise.resolve(true),
            outputStartup(path, application, odataDataSources),
            outputProgram(path, application.name),
            copy(path, 'appsettings.json', isRunning),
            copy(path, 'project.csproj', isRunning),
            copy(path, 'web.config', isRunning)
        ]));
    }
}
function outputIdentity(path, application) {
    var authenticationPath = path_1.join(path, 'Authentication');
    fs_1.ensure(authenticationPath);
    return Promise.all([
        outputIdentityMigrations(path, application),
        writeTemplate(path_1.join(authenticationPath, 'TokenProviderOptions.cs'), 'TokenProviderOptions.cs.ejs', { application: application.name }),
        writeTemplate(path_1.join(authenticationPath, 'TokenProviderMiddleware.cs'), 'TokenProviderMiddleware.cs.ejs', { application: application }),
        writeTemplate(path_1.join(authenticationPath, 'TokenProviderAppBuilderExtensions.cs'), 'TokenProviderAppBuilderExtensions.cs.ejs', { application: application.name }),
        writeTemplate(path_1.join(authenticationPath, 'Bootstrapper.cs'), 'Bootstrapper.cs.ejs', { application: application }),
        writeTemplate(path_1.join(path, 'Controllers', 'RolesController.cs'), 'RolesController.cs.ejs', { application: application }),
        writeTemplate(path_1.join(path, 'Controllers', 'UsersController.cs'), 'UsersController.cs.ejs', { application: application })
    ]);
}
function outputIdentityMigrations(path, application) {
    path = path_1.join(path, 'Data', 'Migrations');
    fs_1.ensure(path);
    return Promise.all([
        writeTemplate(path_1.join(path, 'IdentityModelSnapshot.cs'), 'IdentityModelSnapshot.cs.ejs', { application: application }),
        writeTemplate(path_1.join(path, '00000000000000_CreateIdentitySchema.cs'), '00000000000000_CreateIdentitySchema.cs.ejs', { application: application }),
        writeTemplate(path_1.join(path, '00000000000000_CreateIdentitySchema.Designer.cs'), '00000000000000_CreateIdentitySchema.Designer.cs.ejs', { application: application }),
    ]);
}
function outputStartup(path, application, dataSources) {
    return writeTemplate(path_1.join(path, 'Startup.cs'), 'startup.cs.ejs', { dataSources: dataSources, application: application.name, security: application.security });
}
function outputProgram(path, application) {
    return writeTemplate(path_1.join(path, 'Program.cs'), 'program.cs.ejs', { application: application });
}
function outputODataModels(path, application, dataSource) {
    path = path_1.join(path, 'Models', dataSource.name);
    fs_1.ensure(path);
    return Promise.all(dataSource.schemas.filter(env.validModel).map(function (schema) { return outputODataModel(path, application, schema, dataSource); }));
}
function outputODataControllers(path, application, dataSource) {
    path = path_1.join(path, 'Controllers', dataSource.name);
    fs_1.ensure(path);
    return Promise.all(dataSource.schemas.filter(env.validModel).map(function (schema) { return outputODataController(path, application, schema, dataSource); }));
}
function outputODataModel(path, application, model, dataSource) {
    return writeTemplate(path_1.join(path, model.name + ".cs"), 'model.cs.ejs', {
        model: model,
        application: application,
        dataSource: dataSource
    });
}
function outputODataController(path, application, model, dataSource) {
    return writeTemplate(path_1.join(path, model.entitySet + "Controller.cs"), 'controller.cs.ejs', {
        model: model,
        application: application,
        dataSource: dataSource
    });
}
function outputODataContext(path, application, dataSource) {
    return writeTemplate(path_1.join(path, 'Data', env.type(dataSource.name) + "Context.cs"), 'context.cs.ejs', {
        dataSource: dataSource,
        application: application
    });
}
function designTimeHtml(application) {
    return renderTemplate_1.renderTemplate('app.component.html.ejs', Object.assign({ security: null }, application));
}
exports.designTimeHtml = designTimeHtml;
function setPageTitle(contentDocument, title) {
    contentDocument.querySelector('#title').innerHTML = title;
}
exports.setPageTitle = setPageTitle;
function setLogo(contentDocument, path, application) {
    if (application.logo) {
        var logo = contentDocument.querySelector('#logo img');
        var src = path_1.join(path, 'client', 'src', application.logo);
        logo.src = "file://" + src;
    }
}
exports.setLogo = setLogo;
function navigationContainerElement(contentDocument) {
    return contentDocument.querySelector('#navigation');
}
exports.navigationContainerElement = navigationContainerElement;
function contentElements(contentDocument) {
    return contentDocument.querySelectorAll('#content');
}
exports.contentElements = contentElements;
function copy(path, file, isRunning) {
    if (isRunning === void 0) { isRunning = false; }
    return copyFile(path, file, path_1.basename(file), isRunning);
}
function copyFile(path, file, target, isRunning) {
    if (isRunning === void 0) { isRunning = false; }
    var templates = path_1.join(__dirname, './templates');
    var dest = path_1.join(path, target);
    return fs_1.fileExists(dest).then(function (exists) {
        return !(isRunning && exists);
    }).then(function (write) {
        if (write) {
            return new Promise(function (resolve, reject) {
                var reader = fs.createReadStream(path_1.join(templates, file));
                reader.on('error', reject);
                var writer = fs.createWriteStream(dest);
                writer.on('error', reject);
                reader.pipe(writer);
                resolve();
            });
        }
    });
}
function deleteDataSource(path, name) {
    path = path_1.join(path, 'client', 'src', 'app');
    var filename = env.file(name);
    return Promise.all([
        fs_1.rm(path_1.join(path, filename + ".model.ts")),
        fs_1.rm(path_1.join(path, filename + ".service.ts")),
        fs_1.rm(path_1.join(path, filename + "-auth.service.ts")),
    ]);
}
exports.deleteDataSource = deleteDataSource;
function deletePage(path, name) {
    return fs_1.rm(path_1.join(path, 'client', 'src', 'app', env.file(name)));
}
exports.deletePage = deletePage;
function renamePage(path, from, to) {
    var components = path_1.join(path, 'client', 'src', 'app');
    var fromFile = env.file(from);
    var fromDir = path_1.join(components, fromFile);
    var toFile = env.file(to);
    var toDir = path_1.join(components, toFile);
    return fs.existsSync(fromDir) && fs_1.rename(fromDir, toDir).then(function () {
        var fromFilePath = path_1.join(toDir, fromFile + ".component.ts");
        if (fs.existsSync(fromFilePath)) {
            return fs_1.readFile(fromFilePath, 'utf8').then(function (text) {
                var fromType = "" + env.type(from);
                var toType = "" + env.type(to);
                text = text.replace(new RegExp(fromType + "Generated", 'g'), toType + "Generated")
                    .replace(new RegExp(fromType + "Component", 'g'), toType + "Component")
                    .replace(new RegExp("" + fromFile, 'g'), "" + toFile);
                return fs_1.writeFile(fromFilePath, text, 'utf8');
            }).then(function () { return Promise.all([
                fs_1.rename(fromFilePath, path_1.join(toDir, toFile + ".component.ts")),
                fs_1.rename(path_1.join(toDir, fromFile + "-generated.component.ts"), path_1.join(toDir, toFile + "-generated.component.ts")),
                fs_1.rename(path_1.join(toDir, fromFile + ".component.html"), path_1.join(toDir, toFile + ".component.html")),
            ]); });
        }
    });
}
exports.renamePage = renamePage;
function outputPages(path, application, pages, dataSources) {
    return Promise.all(pages.map(function (page) { return outputPage(path, application, page, dataSources); }));
}
function outputPage(path, application, page, dataSources) {
    return writePage(fs_1.ensure(path_1.join(path, env.file(page.name))), application, page, dataSources);
}
function traverse(events, callback) {
    function descend(actions) {
        return actions.reduce(function (result, action) {
            callback(result, action, descend);
            return result;
        }, []);
    }
    if (!events) {
        return [];
    }
    var actions = Object.keys(events).reduce(function (actions, key) {
        actions.push.apply(actions, events[key]);
        return actions;
    }, []);
    return descend(actions);
}
function setterNames(events) {
    function callback(result, action, traverse) {
        if (Event_1.isSet(action)) {
            result.push(action.name);
        }
        else {
            if (action.then) {
                result.push.apply(result, traverse(action.then));
            }
            if (action.error) {
                result.push.apply(result, traverse(action.error));
            }
        }
    }
    return traverse(events, callback);
}
function serviceNames(events) {
    function callback(result, action, traverse) {
        if (Event_1.isInvoke(action)) {
            if (action.name.includes('.')) {
                var service = action.name.split('.')[0];
                if (service != 'Security') {
                    result.push(service);
                }
            }
        }
        else {
            if (action.then) {
                result.push.apply(result, traverse(action.then));
            }
            if (action.error) {
                result.push.apply(result, traverse(action.error));
            }
        }
    }
    return traverse(events, callback);
}
function pageParameters(pages) {
    var parameters = {};
    function callback(result, action, traverse) {
        if ((Event_1.isPopup(action) || Event_1.isNavigate(action)) && action.parameters) {
            parameters[action.path] = action.parameters.map(function (parameter) { return parameter.name; });
        }
        else {
            if (action.then) {
                traverse(action.then);
            }
            if (action.error) {
                traverse(action.error);
            }
        }
    }
    pages.forEach(function (page) {
        var events = Object.assign({}, page.events);
        page.components.forEach(function (component) { return Object.assign(events, component.events); });
        traverse(events, callback);
    });
    return parameters;
}
function writePage(path, application, page, dataSources) {
    var components = page.components.map(function (component, index) { return Object.assign({ index: index }, component); });
    var types = lodash_1.groupBy(components, 'type');
    var properties = setterNames(page.events).concat('parameters');
    var services = serviceNames(page.events);
    components.forEach(function (component) {
        if (component.events) {
            properties.push.apply(properties, setterNames(component.events));
            services.push.apply(services, serviceNames(component.events));
        }
    });
    var rows = lodash_1.toArray(lodash_1.groupBy(components, 'row')).map(function (row) { return row.map(function (component) {
        var index = types[component.type].indexOf(component);
        return Component_1.prepareComponent(properties, component, index, component.index);
    }); });
    var events = Object.keys(page.events || {}).map(function (name) {
        return {
            name: name,
            value: name,
            actions: page.events[name]
        };
    });
    var data = {
        name: page.name,
        access: page.access,
        events: events,
        rows: rows,
        dataSources: application.security ? dataSources.concat(DataSource_1.swaggerToDataSource(securityDataSource_1.securityDataSource())) : dataSources,
        properties: application.security ? lodash_1.uniq(properties).concat('Security') : lodash_1.uniq(properties),
        security: application.security,
        services: lodash_1.uniq(services),
        componentTypes: lodash_1.uniq(components.map(function (component) { return component.type; })),
        components: lodash_1.flatten(rows)
    };
    var filename = env.file(page.name);
    var promises = [
        writeTemplate(path_1.join(path, filename + "-generated.component.ts"), 'page.generated.ts.ejs', data),
        writeTemplate(path_1.join(path, filename + ".component.html"), 'page.html.ejs', data)
    ];
    if (!fs.existsSync(path_1.join(path, filename + ".component.ts"))) {
        promises.push(writeTemplate(path_1.join(path, filename + ".component.ts"), 'page.ts.ejs', data));
    }
    return Promise.all(promises);
}
function outputDataSources(path, application, services) {
    fs_1.ensure(path);
    return Promise.all(services.map(function (service) { return outputService(path, application, service); }).slice());
}
function outputSecurity(path, application) {
    var _a = application.security.roles, roles = _a === void 0 ? [] : _a;
    var roleGuards = roles.map(function (role) { return writeTemplate(path_1.join(path, env.file(role) + ".guard.ts"), 'guard.ts.ejs', { role: role }); });
    return Promise.all(roleGuards.concat([
        writeTemplate(path_1.join(path, 'unauthorized.component.ts'), 'unauthorized.ts.ejs', application),
        writeTemplate(path_1.join(path, 'unauthorized.component.html'), 'unauthorized.html.ejs', application),
        writeTemplate(path_1.join(path, 'security.service.ts'), 'security.service.ts.ejs', application),
        writeTemplate(path_1.join(path, 'auth.guard.ts'), 'auth.guard.ts.ejs', application)
    ]));
}
function outputService(path, application, service) {
    return Promise.all([
        writeModels(path, service),
        writeService(path, application, service),
        writeAuth(path, service)
    ]);
}
function writeAppModule(path, application, dataSources, pages) {
    return writeTemplate(path_1.join(path, 'app.module.ts'), 'app.module.ts.ejs', {
        theme: application.theme,
        application: application,
        dataSources: dataSources,
        pages: pages
    });
}
function writeAuth(path, service) {
    if (service.authorization) {
        var data = Object.assign({ name: service.name }, service.authorization);
        return writeTemplate(path_1.join(path, env.file(service.name) + "-auth.service.ts"), 'auth.ts.ejs', data);
    }
}
function writeService(path, application, service) {
    return writeTemplate(path_1.join(path, env.file(service.name) + ".service.ts"), 'service.ts.ejs', Object.assign({ security: application.security }, service));
}
function writeModels(path, service) {
    return writeTemplate(path_1.join(path, env.file(service.name) + ".model.ts"), 'model.ts.ejs', service);
}
function writeTemplate(path, template, data) {
    return renderTemplate_1.renderTemplate(template, data).then(function (output) { return updateFile(path, output); });
}
//# sourceMappingURL=index.js.map