"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var platform_browser_1 = require("@angular/platform-browser");
// https://github.com/angular/angular/blob/master/packages/platform-browser/src/dom/dom_renderer.ts
var NAMESPACE_URIS = {
    'svg': 'http://www.w3.org/2000/svg',
    'xhtml': 'http://www.w3.org/1999/xhtml',
    'xlink': 'http://www.w3.org/1999/xlink',
    'xml': 'http://www.w3.org/XML/1998/namespace',
    'xmlns': 'http://www.w3.org/2000/xmlns/',
};
function decoratePreventDefault(eventHandler) {
    return function (event) {
        var allowDefaultBehavior = eventHandler(event);
        if (allowDefaultBehavior === false) {
            event.preventDefault();
            event.returnValue = false;
        }
    };
}
exports.DOCUMENT_TOKEN = new core_1.InjectionToken('document');
var RendererFactory = (function () {
    function RendererFactory(document, eventManager, sharedStylesHost) {
        this.sharedStylesHost = sharedStylesHost;
        this.rendererByCompId = new Map();
        this.defaultRenderer = new Renderer(document, eventManager);
    }
    ;
    RendererFactory.prototype.createRenderer = function (element, type) {
        if (!element || !type) {
            return this.defaultRenderer;
        }
        switch (type.encapsulation) {
            default: {
                if (!this.rendererByCompId.has(type.id)) {
                    var styles = platform_browser_1.ɵflattenStyles(type.id, type.styles, []);
                    this.sharedStylesHost.addStyles(styles);
                    this.rendererByCompId.set(type.id, this.defaultRenderer);
                }
                return this.defaultRenderer;
            }
        }
    };
    return RendererFactory;
}());
RendererFactory = __decorate([
    core_1.Injectable(),
    __param(0, core_1.Inject(exports.DOCUMENT_TOKEN)),
    __metadata("design:paramtypes", [Object, platform_browser_1.EventManager, platform_browser_1.ɵDomSharedStylesHost])
], RendererFactory);
exports.RendererFactory = RendererFactory;
var Renderer = (function () {
    function Renderer(document, eventManager) {
        this.document = document;
        this.eventManager = eventManager;
        this.data = Object.create(null);
    }
    Renderer.prototype.destroy = function () { };
    Renderer.prototype.createElement = function (name, namespace) {
        if (namespace) {
            return this.document.createElementNS(NAMESPACE_URIS[namespace], name);
        }
        return this.document.createElement(name);
    };
    Renderer.prototype.createComment = function (value) { return this.document.createComment(value); };
    Renderer.prototype.createText = function (value) { return this.document.createTextNode(value); };
    Renderer.prototype.appendChild = function (parent, newChild) { parent.appendChild(newChild); };
    Renderer.prototype.insertBefore = function (parent, newChild, refChild) {
        if (parent) {
            parent.insertBefore(newChild, refChild);
        }
    };
    Renderer.prototype.removeChild = function (parent, oldChild) {
        if (parent) {
            parent.removeChild(oldChild);
        }
    };
    Renderer.prototype.selectRootElement = function (selectorOrNode) {
        var el = typeof selectorOrNode === 'string' ? this.document.querySelector(selectorOrNode) :
            selectorOrNode;
        if (!el) {
            throw new Error("The selector \"" + selectorOrNode + "\" did not match any elements");
        }
        el.textContent = '';
        return el;
    };
    Renderer.prototype.parentNode = function (node) { return node.parentNode; };
    Renderer.prototype.nextSibling = function (node) { return node.nextSibling; };
    Renderer.prototype.setAttribute = function (el, name, value, namespace) {
        if (namespace) {
            name = namespace + ":" + name;
            var namespaceUri = NAMESPACE_URIS[namespace];
            if (namespaceUri) {
                el.setAttributeNS(namespaceUri, name, value);
            }
            else {
                el.setAttribute(name, value);
            }
        }
        else {
            el.setAttribute(name, value);
        }
    };
    Renderer.prototype.removeAttribute = function (el, name, namespace) {
        if (namespace) {
            var namespaceUri = NAMESPACE_URIS[namespace];
            if (namespaceUri) {
                el.removeAttributeNS(namespaceUri, name);
            }
            else {
                el.removeAttribute(namespace + ":" + name);
            }
        }
        else {
            el.removeAttribute(name);
        }
    };
    Renderer.prototype.addClass = function (el, name) { el.classList.add(name); };
    Renderer.prototype.removeClass = function (el, name) { el.classList.remove(name); };
    Renderer.prototype.setStyle = function (el, style, value, flags) {
        if (flags & core_1.RendererStyleFlags2.DashCase) {
            el.style.setProperty(style, value, !!(flags & core_1.RendererStyleFlags2.Important) ? 'important' : '');
        }
        else {
            el.style[style] = value;
        }
    };
    Renderer.prototype.removeStyle = function (el, style, flags) {
        if (flags & core_1.RendererStyleFlags2.DashCase) {
            el.style.removeProperty(style);
        }
        else {
            // IE requires '' instead of null
            // see https://github.com/angular/angular/issues/7916
            el.style[style] = '';
        }
    };
    Renderer.prototype.setProperty = function (el, name, value) {
        el[name] = value;
    };
    Renderer.prototype.setValue = function (node, value) { node.nodeValue = value; };
    Renderer.prototype.listen = function (target, event, callback) {
        if (typeof target === 'string') {
            return this.eventManager.addGlobalEventListener(target, event, decoratePreventDefault(callback));
        }
        return this.eventManager.addEventListener(target, event, decoratePreventDefault(callback));
    };
    return Renderer;
}());
//# sourceMappingURL=renderer-factory.js.map