"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var lodash_1 = require("lodash");
var ChartComponent = (function () {
    function ChartComponent() {
        this.select = new core_1.EventEmitter();
        this.results = [];
        this.scheme = {
            domain: [
                '#2976C2',
                '#20A85B',
                '#FFB300',
                '#FF4548',
                '#844CDA',
                '#2C9AB7',
                '#A0E4CD',
                '#FFD66B',
                '#F37F57',
                '#38545E',
                '#9B4B6D',
                '#FD6A92',
                '#4A4069',
                '#293853',
                '#6C95FF',
                '#45CDCF'
            ]
        };
    }
    ChartComponent.prototype.update = function () {
        var _this = this;
        if (this.data && this.categoryProperty && this.valueProperty) {
            var series = void 0;
            if (this.series) {
                var groups = lodash_1.groupBy(this.data, this.series);
                series = lodash_1.map(groups, function (data, name) {
                    var series = _this.map(data);
                    return {
                        name: name,
                        series: series
                    };
                });
            }
            else {
                series = this.map(this.data);
            }
            if ((this.chartType == 'Line' || this.chartType == 'Area') && !this.series) {
                this.results = [
                    {
                        name: this.categoryProperty,
                        series: series
                    }
                ];
            }
            else {
                this.results = series;
            }
        }
    };
    ChartComponent.prototype.map = function (data) {
        var _this = this;
        return data.map(function (item) {
            return {
                name: lodash_1.get(item, _this.categoryProperty),
                value: +lodash_1.get(item, _this.valueProperty)
            };
        });
    };
    ChartComponent.prototype.ngOnInit = function () {
        this.update();
    };
    ChartComponent.prototype.ngOnChanges = function () {
        this.update();
    };
    return ChartComponent;
}());
ChartComponent.decorators = [
    { type: core_1.Component, args: [{
                selector: 'rz-chart',
                host: {
                    style: 'display: block; height: 100%'
                },
                template: "<rz-container [title]=\"title\">\n    <div #container style=\"width: 100%; min-height: 200px; height: 100%\">\n      <ngx-charts-bar-vertical-2d\n        *ngIf=\"chartType == 'Column' && series\"\n        [view]=\"[container.clientWidth, container.clientHeight]\"\n        [scheme]=\"scheme\"\n        xAxis=\"true\"\n        yAxis=\"true\"\n        [results]=\"results\"\n        [legend]=\"legend\"\n      ></ngx-charts-bar-vertical-2d>\n      <ngx-charts-bar-vertical\n        *ngIf=\"chartType == 'Column' && !series\"\n        [view]=\"[container.clientWidth, container.clientHeight]\"\n        [scheme]=\"scheme\"\n        xAxis=\"true\"\n        yAxis=\"true\"\n        [results]=\"results\"\n        [legend]=\"legend\"\n      ></ngx-charts-bar-vertical>\n      <ngx-charts-bar-horizontal-2d\n        *ngIf=\"chartType == 'Bar' && series\"\n        [view]=\"[container.clientWidth, container.clientHeight]\"\n        [scheme]=\"scheme\"\n        xAxis=\"true\"\n        yAxis=\"true\"\n        [results]=\"results\"\n        [legend]=\"legend\"\n      ></ngx-charts-bar-horizontal-2d>\n      <ngx-charts-bar-horizontal\n        *ngIf=\"chartType == 'Bar' && !series\"\n        [view]=\"[container.clientWidth, container.clientHeight]\"\n        [scheme]=\"scheme\"\n        xAxis=\"true\"\n        yAxis=\"true\"\n        [results]=\"results\"\n        [legend]=\"legend\"\n      ></ngx-charts-bar-horizontal>\n      <ngx-charts-line-chart\n        *ngIf=\"chartType == 'Line'\"\n        [view]=\"[container.clientWidth, container.clientHeight]\"\n        [scheme]=\"scheme\"\n        xAxis=\"true\"\n        yAxis=\"true\"\n        [results]=\"results\"\n        [legend]=\"legend\"\n      ></ngx-charts-line-chart>\n    </div>\n  </rz-container>"
            },] },
];
/** @nocollapse */
ChartComponent.ctorParameters = function () { return []; };
ChartComponent.propDecorators = {
    'title': [{ type: core_1.Input },],
    'series': [{ type: core_1.Input },],
    'legend': [{ type: core_1.Input },],
    'chartType': [{ type: core_1.Input },],
    'data': [{ type: core_1.Input },],
    'categoryProperty': [{ type: core_1.Input },],
    'valueProperty': [{ type: core_1.Input },],
    'select': [{ type: core_1.Output },],
};
exports.ChartComponent = ChartComponent;
