"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var forms_1 = require("@angular/forms");
var lodash_1 = require("lodash");
var DROPDOWN_VALUE_ACCESSOR = {
    provide: forms_1.NG_VALUE_ACCESSOR,
    useExisting: core_1.forwardRef(function () { return DropDownComponent; }),
    multi: true
};
var DropDownComponent = (function () {
    function DropDownComponent() {
        this.change = new core_1.EventEmitter();
        this.onChange = function (_) { };
        this.onTouched = function () { };
    }
    Object.defineProperty(DropDownComponent.prototype, "value", {
        get: function () {
            return this.innerValue;
        },
        set: function (value) {
            if (value != this.innerValue) {
                this.innerValue = value;
                this.onChange(value);
                this.change.next(value);
            }
        },
        enumerable: true,
        configurable: true
    });
    DropDownComponent.prototype.text = function (item) {
        return this.textProperty ? lodash_1.get(item, this.textProperty) : item;
    };
    DropDownComponent.prototype.writeValue = function (value) {
        this.innerValue = value;
    };
    DropDownComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    DropDownComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    DropDownComponent.prototype.ngOnChanges = function (changes) {
        if (changes.hasOwnProperty('data') || changes.hasOwnProperty('textProperty')) {
            this.setOptions();
        }
    };
    DropDownComponent.prototype.setOptions = function () {
        var _this = this;
        if (!this.data) {
            return;
        }
        this.options = this.data.map(function (item) {
            return {
                label: _this.textProperty ? lodash_1.get(item, _this.textProperty) : item,
                value: item
            };
        });
    };
    return DropDownComponent;
}());
DropDownComponent.decorators = [
    { type: core_1.Component, args: [{
                selector: 'rz-dropdown',
                template: "\n  <p-dropdown\n    (blur)=\"onTouched()\"\n    [(ngModel)]=\"value\"\n    [autoWidth]=\"false\"\n    [options]=\"options\"\n    [placeholder]=\"placeholder\">\n  </p-dropdown>\n  ",
                providers: [
                    DROPDOWN_VALUE_ACCESSOR
                ]
            },] },
];
/** @nocollapse */
DropDownComponent.ctorParameters = function () { return []; };
DropDownComponent.propDecorators = {
    'data': [{ type: core_1.Input },],
    'placeholder': [{ type: core_1.Input },],
    'textProperty': [{ type: core_1.Input },],
    'change': [{ type: core_1.Output },],
    'value': [{ type: core_1.Input },],
};
exports.DropDownComponent = DropDownComponent;
