"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var lodash_1 = require("lodash");
var form_component_1 = require("./form.component");
var FormFieldComponent = (function () {
    function FormFieldComponent(parent) {
        this.type = 'string';
        this.parent = parent;
    }
    FormFieldComponent.prototype.ngOnChanges = function (changes) {
        if (changes.hasOwnProperty('data') ||
            changes.hasOwnProperty('textProperty') ||
            changes.hasOwnProperty('valueProperty')) {
            this.setOptions();
        }
    };
    FormFieldComponent.prototype.setOptions = function () {
        var _this = this;
        if (!this.data) {
            return;
        }
        this.options = this.data.map(function (item) {
            return {
                label: _this.textProperty ? lodash_1.get(item, _this.textProperty) : item,
                value: _this.valueProperty ? lodash_1.get(item, _this.valueProperty) : item
            };
        });
    };
    return FormFieldComponent;
}());
FormFieldComponent.decorators = [
    { type: core_1.Component, args: [{
                selector: 'rz-form-field',
                template: "\n  <div class=\"row form-group ui-fluid\" *ngIf=\"parent.form?.controls[property]\">\n    <label class=\"col-sm-3 col-form-label\" [attr.for]=\"property\">{{ title }}</label>\n    <div class=\"col\">\n      <select *ngIf=\"data\" [attr.id]=\"property\" [formControl]=\"parent.form.controls[property]\" class=\"form-control\">\n         <option *ngIf=\"placeholder\" disabled selected hidden [ngValue]=\"null\">{{ placeholder }}</option>\n         <option *ngFor=\"let option of options\" [ngValue]=\"option.value\">{{ option.label }}</option>\n      </select>\n      <p-spinner *ngIf=\"type == 'integer' && !data\"\n        [placeholder]=\"placeholder\"\n        step=\"1\"\n        class=\"form-control\"\n        [formControl]=\"parent.form.controls[property]\"\n        [inputId]=\"property\">\n      </p-spinner>\n      <p-spinner *ngIf=\"type == 'number' && !data\"\n        class=\"form-control\"\n        [formControl]=\"parent.form.controls[property]\"\n        [attr.placeholder]=\"placeholder\"\n        [inputId]=\"property\">\n      </p-spinner>\n      <p-calendar appendTo=\"body\" *ngIf=\"type == 'string' && format == 'date-time'\"\n        class=\"form-control\"\n        [showIcon]=\"true\"\n        [formControl]=\"parent.form.controls[property]\"\n        [placeholder]=\"placeholder\"\n        [inputId]=\"property\">\n      </p-calendar>\n      <p-checkbox *ngIf=\"type == 'boolean' && !required\"\n        [formControl]=\"parent.form.controls[property]\"\n        [inputId]=\"property\"\n        [binary]=\"true\">\n      </p-checkbox>\n      <p-triStateCheckbox *ngIf=\"type == 'boolean' && required\"\n        [inputId]=\"property\"\n        [formControl]=\"parent.form.controls[property]\">\n      </p-triStateCheckbox>\n      <input *ngIf=\"type == 'string' && format == 'password'\"\n        class=\"form-control\"\n        type=\"password\"\n        [formControl]=\"parent.form.controls[property]\"\n        [attr.placeholder]=\"placeholder\"\n        [attr.id]=\"property\">\n      <input *ngIf=\"type == 'string' && !format && !data\"\n        class=\"form-control\"\n        [formControl]=\"parent.form.controls[property]\"\n        [attr.placeholder]=\"placeholder\"\n        [attr.id]=\"property\">\n      <div class=\"md-inputfield\"\n        *ngIf=\"(parent.form.controls[property].touched || parent.submitted) && parent.form.controls[property].errors?.required\">\n        <div class=\"ui-message ui-messages-error ui-corner-all\">\n          {{ title }} is required\n        </div>\n      </div>\n    </div>\n  </div>\n  "
            },] },
];
/** @nocollapse */
FormFieldComponent.ctorParameters = function () { return [
    { type: form_component_1.FormComponent, decorators: [{ type: core_1.Host },] },
]; };
FormFieldComponent.propDecorators = {
    'property': [{ type: core_1.Input },],
    'title': [{ type: core_1.Input },],
    'type': [{ type: core_1.Input },],
    'format': [{ type: core_1.Input },],
    'textProperty': [{ type: core_1.Input },],
    'valueProperty': [{ type: core_1.Input },],
    'placeholder': [{ type: core_1.Input },],
    'data': [{ type: core_1.Input },],
    'required': [{ type: core_1.Input },],
};
exports.FormFieldComponent = FormFieldComponent;
