"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var forms_1 = require("@angular/forms");
var lodash_1 = require("lodash");
var form_field_component_1 = require("./form-field.component");
var FormComponent = (function () {
    function FormComponent(formBuilder) {
        this.formBuilder = formBuilder;
        this.visible = true;
        this.submitText = 'Save';
        this.cancelText = 'Cancel';
        this.submit = new core_1.EventEmitter();
        this.cancel = new core_1.EventEmitter();
        this.form = new forms_1.FormGroup({});
    }
    Object.defineProperty(FormComponent.prototype, "data", {
        set: function (data) {
            this.value = data;
            if (this.fields != undefined) {
                this.createForm();
            }
        },
        enumerable: true,
        configurable: true
    });
    FormComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        this.createForm();
        this.subscription = this.fields.changes.subscribe(function () { return _this.createForm(); });
    };
    FormComponent.prototype.createForm = function () {
        var _this = this;
        var form = this.fields.reduce(function (form, field) {
            var value = lodash_1.get(_this.value, field.property, null);
            if (value != null && field.type == 'string' && field.format == 'date-time') {
                value = new Date(Date.parse(value));
            }
            if (field.required) {
                form[field.property] = [value, forms_1.Validators.required];
            }
            else {
                form[field.property] = value;
            }
            return form;
        }, {});
        this.form = this.formBuilder.group(form);
    };
    FormComponent.prototype.ngOnDestroy = function () {
        this.subscription.unsubscribe();
    };
    FormComponent.prototype.onSubmit = function (event) {
        event.preventDefault();
        event.stopPropagation();
        this.submitted = true;
        if (this.form.valid) {
            if (this.value) {
                Object.assign(this.value, this.form.value);
            }
            this.submit.next(this.form.value);
        }
    };
    FormComponent.prototype.onCancel = function () {
        this.cancel.next();
    };
    return FormComponent;
}());
FormComponent.decorators = [
    { type: core_1.Component, args: [{
                selector: 'rz-form',
                template: "\n  <rz-container *ngIf=\"visible\" [title]=\"title\">\n    <form [formGroup]=\"form\" (submit)=\"onSubmit($event)\">\n      <ng-content></ng-content>\n      <div class=\"form-group row\">\n        <div class=\"offset-sm-3 col\">\n          <button type=\"submit\" pButton icon=\"ui-icon-save\" [label]=\"submitText\"></button>\n          <a (click)=\"onCancel()\">{{cancelText}}</a>\n        </div>\n      </div>\n    </form>\n  </rz-container>"
            },] },
];
/** @nocollapse */
FormComponent.ctorParameters = function () { return [
    { type: forms_1.FormBuilder, },
]; };
FormComponent.propDecorators = {
    'data': [{ type: core_1.Input },],
    'title': [{ type: core_1.Input },],
    'visible': [{ type: core_1.Input },],
    'submitText': [{ type: core_1.Input },],
    'cancelText': [{ type: core_1.Input },],
    'submit': [{ type: core_1.Output },],
    'cancel': [{ type: core_1.Output },],
    'fields': [{ type: core_1.ContentChildren, args: [form_field_component_1.FormFieldComponent,] },],
};
exports.FormComponent = FormComponent;
