"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var GridColumn = (function () {
    function GridColumn() {
    }
    return GridColumn;
}());
GridColumn.decorators = [
    { type: core_1.Component, args: [{
                selector: 'rz-grid-column',
                template: ''
            },] },
];
/** @nocollapse */
GridColumn.ctorParameters = function () { return []; };
GridColumn.propDecorators = {
    'title': [{ type: core_1.Input },],
    'type': [{ type: core_1.Input },],
    'format': [{ type: core_1.Input },],
    'property': [{ type: core_1.Input },],
    'template': [{ type: core_1.ContentChild, args: [core_1.TemplateRef,] },],
};
exports.GridColumn = GridColumn;
var GridComponent = (function () {
    function GridComponent() {
        this.allowDelete = false;
        this.allowAdd = false;
        this.allowSorting = false;
        this.allowMultiSorting = false;
        this.allowPaging = false;
        this.allowFiltering = false;
        this.allowScrolling = false;
        this.pageSize = 10;
        this.select = new core_1.EventEmitter();
        this.delete = new core_1.EventEmitter();
        this.add = new core_1.EventEmitter();
        this.loadData = new core_1.EventEmitter();
        this.multiSortMeta = [];
    }
    GridComponent.prototype.onSelect = function (value) {
        this.select.next(value);
    };
    GridComponent.prototype.onDelete = function (value) {
        this.delete.next(value);
        this.data.splice(this.data.indexOf(value), 1);
    };
    GridComponent.prototype.onAdd = function (value) {
        this.add.next(value);
    };
    GridComponent.prototype.onLazyLoad = function (event) {
        if (this.allowSorting && event.sortField) {
            var field = event.multiSortMeta.find(function (i) { return i && i.field == event.sortField; });
            if (!this.allowMultiSorting) {
                event.multiSortMeta.splice(event.multiSortMeta.indexOf(field), 1);
                field = null;
            }
            if (field) {
                if (field.order == -1 && event.sortOrder == 1) {
                    event.multiSortMeta.splice(event.multiSortMeta.indexOf(field), 1);
                }
                else {
                    field.order = event.sortOrder;
                }
            }
            else {
                event.multiSortMeta.push({ field: event.sortField, order: event.sortOrder });
            }
        }
        if (this.allowSorting || this.allowPaging || this.allowFiltering) {
            // Design-time error
            try {
                this.loadData.next({
                    skip: this.allowPaging ? event.first : null,
                    top: this.allowPaging ? event.rows : null,
                    orderby: this.allowSorting ? orderByToString(event.multiSortMeta) : '',
                    filter: this.allowFiltering ? filtersToString(event.filters, this.columns.toArray()) : ''
                });
            }
            catch (e) {
                //
            }
        }
    };
    return GridComponent;
}());
GridComponent.decorators = [
    { type: core_1.Component, args: [{
                selector: 'rz-grid',
                host: {
                    style: 'display: block; height: 100%',
                },
                template: "<rz-container [title]=\"title\">\n    <p-dataTable\n      *ngIf=\"data\"\n      selectionMode=\"single\"\n      [scrollable]=\"allowScrolling\"\n      scrollHeight=\"100%\"\n      scrollWidth=\"100%\"\n      [value]=\"data\"\n      [totalRecords]=\"count\"\n      [lazy]=\"true\"\n      (onLazyLoad)=\"onLazyLoad($event)\"\n      [sortMode]=\"allowMultiSorting ? 'multiple' : 'single'\"\n      [multiSortMeta]=\"multiSortMeta\"\n      (onRowSelect)=\"onSelect($event.data)\"\n      [rows]=\"allowPaging ? pageSize : -1\" [paginator]=\"allowPaging\"\n    >\n      <p-header *ngIf=\"allowAdd\">\n        <button type=\"button\" pButton icon=\"fa-plus\" (click)=\"onAdd(data)\" label=\"Add\"></button>\n      </p-header>\n      <p-column *ngFor=\"let column of columns; let index = index\" [field]=\"column.property\" [header]=\"column.title\"\n        [sortable]=\"allowSorting ? 'custom': false\" [filter]=\"allowFiltering\">\n        <ng-template *ngIf=\"column.template\" pTemplate=\"body\" let-data=\"rowData\">\n          <ng-template [ngTemplateOutlet]=\"column.template\" [ngOutletContext]=\"{ $implicit: data }\">\n          </ng-template>\n        </ng-template>\n      </p-column>\n      <p-column *ngIf=\"allowDelete\" [style]=\"{'width': '70px'}\">\n        <ng-template let-data=\"rowData\" pTemplate=\"body\">\n          <button type=\"button\" pButton class=\"red-btn\" icon=\"ui-icon-trash\" (click)=\"onDelete(data)\"></button>\n        </ng-template>\n      </p-column>\n    </p-dataTable>\n    </rz-container>"
            },] },
];
/** @nocollapse */
GridComponent.ctorParameters = function () { return []; };
GridComponent.propDecorators = {
    'data': [{ type: core_1.Input },],
    'allowDelete': [{ type: core_1.Input },],
    'allowAdd': [{ type: core_1.Input },],
    'allowSorting': [{ type: core_1.Input },],
    'allowMultiSorting': [{ type: core_1.Input },],
    'allowPaging': [{ type: core_1.Input },],
    'allowFiltering': [{ type: core_1.Input },],
    'allowScrolling': [{ type: core_1.Input },],
    'pageSize': [{ type: core_1.Input },],
    'count': [{ type: core_1.Input },],
    'title': [{ type: core_1.Input },],
    'select': [{ type: core_1.Output },],
    'delete': [{ type: core_1.Output },],
    'add': [{ type: core_1.Output },],
    'loadData': [{ type: core_1.Output },],
    'columns': [{ type: core_1.ContentChildren, args: [GridColumn,] },],
};
exports.GridComponent = GridComponent;
function orderByToString(multiSortMeta) {
    return multiSortMeta.map(function (i) { return "" + i.field + (i.order == -1 ? ' desc' : ''); }).join(',');
}
function filtersToString(filters, columns) {
    return Object.keys(filters).map(function (key) {
        return filterToString(key, filters[key], columns.find(function (c) { return c.property == key; }));
    }).filter(function (i) { return i != null; }).join(' and ');
}
function filterToString(key, filter, column) {
    if (filter.value != '' && column && column.type) {
        if (column.type == 'string') {
            if (column.format == 'date-time' && (new Date(filter.value) instanceof Date)) {
                return key + " gt cast(" + filter.value + ",Edm.DateTimeOffset)";
            }
            else {
                return "contains(" + key + ", '" + filter.value + "')";
            }
        }
        else if (column.type == 'number' && !isNaN(+filter.value)) {
            return key + " eq " + filter.value;
        }
        else if (column.type == 'boolean' &&
            (filter.value.toLowerCase() == 'true' || filter.value.toLowerCase() == 'false')) {
            return key + " eq " + filter.value;
        }
        else {
            return null;
        }
    }
    return null;
}
